/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */

#[macro_use]
extern crate serde_derive;
extern crate toml;

// use serde::de::Deserialize;
use std::collections::HashMap;
use std::io;
use std::path::PathBuf;

#[derive(Deserialize, Debug)]
struct Config {
    default: String,
    browsers: HashMap<String, Vec<String>>,
    overrides: HashMap<String, Vec<String>>,
}

impl Config {
    fn locate() -> io::Result<PathBuf> {
        use std::env;

        // first priority: if a config file exists in the same directory
        let mut paths = vec![PathBuf::from(".")];

        // second priority: in the user's config dir
        match env::var("XDG_CONFIG_HOME") {
            Ok(confdir) => paths.push(PathBuf::from(&confdir).join("choosier")),
            Err(_) => {
                if let Ok(homedir) = env::var("HOME") {
                    paths.push(PathBuf::from(&homedir).join(".config").join("choosier"))
                }
            }
        }

        // third priority: in /etc
        paths.push(PathBuf::from("/etc/choosier"));

        for p in paths {
            let cfg_path = p.join("choosier.toml");
            if cfg_path.exists() {
                return Ok(cfg_path);
            }
        }
        Err(io::Error::new(
            io::ErrorKind::NotFound,
            "configuration file not found",
        ))
    }

    fn parse(config_str: &str) -> Result<Config, io::Error> {
        let config: Config = toml::from_str(&config_str)?;
        Ok(config)
    }

    fn read() -> Result<String, io::Error> {
        use std::fs::File;
        use std::io::prelude::*;

        let cfg_path = Config::locate()?;
        let mut file = File::open(cfg_path)?;
        let mut contents = String::new();
        file.read_to_string(&mut contents)?;
        Ok(contents)
    }
}

fn open_url(url: &str, config: Config) {
    use std::process::Command;

    let mut pick = config.default;

    for (browser, urls) in config.overrides {
        for u in urls {
            if url.find(&u).is_some() {
                pick = browser.to_string();
            }
        }
    }

    let cmds = &config.browsers[&pick];
    match Command::new(&cmds[0]).args(&cmds[1..]).arg(url).spawn() {
        Ok(_) => (),
        Err(e) => {
            println!("Failed to launch browser: {}", e);
        }
    }
}

fn main() {
    use std::env;

    let args: Vec<String> = env::args().collect();
    if args.len() != 2 {
        println!("Usage: {} URL", args[0]);
        return;
    }
    let url = &args[1];

    let config_str = Config::read().expect("Unable to read configuration file");
    let config = Config::parse(&config_str).expect("Unable to parse configuration");
    open_url(url, config);
}
